#ifndef Array_hh
#define Array_hh


#include "Random.hh"
#include <assert.h>
#include <iostream.h>


template <class T> 
class Array {

	private:		
		int n;		// Number of elements
		int lo;		// Lower index
		int hi;		// Higher index
		T* a;		// The "real" array
		
	public:
	
		inline Array (const int sze=0) {
			assert(sze>=0);
			n=sze;
			lo=0;
			hi=sze-1;
			if (n==0) {
				a=0; 
			} else {
				a=new T[n];
		}	}
		
		inline Array (const int low, const int high) {			
			assert(high>=low);
			n=high-low+1;
			lo=low;
			hi=high;
			a=new T[n];
		}
		
		inline Array (const Array<T>& src) {
			n=src.n;
			lo=src.lo;
			hi=src.hi;
			if (src.a) {
				a=new T[n];
				for (int i=0; i<n; i++) {	
					a[i]=src.a[i];
				}
			} else {
				a=0;
		}	}
		
		inline ~Array () {
			if (a) {
				delete[] a;
		}	}
		
		inline Array<T>& operator= (const Array<T>& src) {
			if (&src!=this) {
				if (n!=src.n) {
					delete[] a;
					n=src.n;
					if (src.a) {
						a=new T[n];
					} else {
						a=0;
					}
				}
				lo=src.lo;
				hi=src.hi;
				for (int i=0; i<n; i++) {	
					a[i]=src.a[i];
			}	}
			return *this;
		}
		
		inline const T& operator[] (const int i) const {
			assert(i>=lo);
			assert(i<=hi);
			return a[i-lo];
		}

		inline T& operator[] (const int i) {
			assert(i>=lo);
			assert(i<=hi);
			return a[i-lo];
		}
		
		inline void swap (const int i, const int j) {
			assert(i>=lo);
			assert(i<=hi);
			const T x=a[i-lo];
			a[i-lo]=a[j-lo];
			a[j-lo]=x;
		}
		
		inline void permute () {
			for (int i=lo; i<hi; i++) {
				const int j=Random(i,hi);
				swap(i,j);
			}
		}
		
		void sort (const int left, const int right) {
			if (right>left) {
				T t,v;
				int i,j;
				v=a[right-lo]; i=left-1; j=right;
				do {
					do {i++;} while (a[i-lo]<v);
					do {j--;} while (a[j-lo]>v);
					t=a[i-lo]; a[i-lo]=a[j-lo]; a[j-lo]=t;
				} while (j>i);
				a[j-lo]=a[i-lo]; a[i-lo]=a[right-lo]; a[right-lo]=t;
				sort(left,i-1);
				sort(i+1,right);
		}	}
				
		inline void sort (void) {
			sort(lo,hi);
		}
		
		inline int size () const {
			return n;
		}

		inline int low () const {
			return lo;
		}

		inline int high () const {
			return n;
		}

		friend ostream& operator<< (ostream& s, const Array<T>& src) {
			for (int i=0; i<src.n; i++) {
				s << " " << src.a[i];
			}
			return s;
		}	

		friend istream& operator>> (istream& s, Array<T>& src) {
			for (int i=0; i<src.n; i++) {
				s >> src.a[i];
			}
			return s;
		}			
};




#endif
