#ifndef Array2_hh
#define Array2_hh


#include <assert.h>
#include <iostream.h>


template <class T> 
class Array2 {

	private:		
		int lo1;	// Lower index for dimension 1
		int hi1;	// Higher index for dimension 1
		int lo2;	// Lower index for dimension 2
		int hi2;	// Higher index for dimension 2
		int di1;	// Dimension 1
		int di2;	// Dimension 2
		int n;		// Total number of elements
		T* a;		// The "real" array
		
	public:
	
		inline Array2 (const int dim1, const int dim2) {
			assert(dim1>=0);
			assert(dim2>=0);
			lo1=0;
			lo2=0;
			hi1=dim1-1;
			hi2=dim2-1;
			di1=dim1;
			di2=dim2;
			n=di1*di2;
			if (n==0) {
				a=0; 
			} else {
				a=new T[n];
		}	}
		
		inline Array2 (const int low1, const int high1, const int low2, const int high2) {
			assert(low1>=high1);
			assert(low2>=high2);
			lo1=low1;
			lo2=low2;
			hi1=high1;
			hi2=high2;
			di1=hi1-lo1+1;
			di2=hi2-lo2+1;
			n=di1*di2;
			if (n==0) {
				a=0; 
			} else {
				a=new T[n];
		}	}
		
		inline Array2 (const Array2<T>& src) {
			n=src.n;
			lo1=src.lo1;
			lo2=src.lo2;
			hi1=src.hi1;
			hi2=src.hi2;
			di1=src.d1;
			di2=src.di2;
			n=src.n;
			if (src.a) {
				a=new T[n];
				for (int i=0; i<n; i++) {	
					a[i]=src.a[i];
				}
			} else {
				a=0;
		}	}
		
		inline ~Array2 () {
			if (a) {
				delete[] a;
		}	}
		
		inline Array2<T>& operator= (const Array2<T>& src) {
			if (&src!=this) {
				if (n!=src.n) {
					delete[] a;
					n=src.n;
					if (src.a) {
						a=new T[n];
					} else {
						a=0;
					}
				}
				lo1=src.lo1;
				lo2=src.lo2;
				hi1=src.hi1;
				hi2=src.hi2;
				di1=src.di1;
				di2=src.di2;
				for (int i=0; i<n; i++) {	
					a[i]=src.a[i];
			}	}
			return *this;
		}
		
		inline const T& operator() (const int i, const int j) const {
			assert(i>=lo1);
			assert(i<=hi1);
			assert(j>=lo2);
			assert(j<=hi2);
			return a[(i-lo1)*di2+(j-lo2)];
		}

		inline T& operator() (const int i1, const int i2) {
			assert(i1>=lo1);
			assert(i1<=hi1);
			assert(i2>=lo2);
			assert(i2<=hi2);
			return a[(i1-lo1)*di2+(i2-lo2)];
		}
		
		inline int low1 () const {
			return lo1;
		}

		inline int low2 () const {
			return lo2;
		}

		inline int high1 () const {
			return hi1;
		}

		inline int high2 () const {
			return hi2;
		}

};




#endif
